@@ -2,6 +2,9 @@ from hydrafloods.datasets import *
 from hydrafloods.geeutils import *
 from hydrafloods.thresholding import *
 from hydrafloods.filtering import *
+from hydrafloods.corrections import *
+from hydrafloods.indices import *
+from hydrafloods import fetch, utils
 # from hydrafloods import *
 
-__version__ = "0.2.0"
\ No newline at end of file
+__version__ = "0.3.8"
\ No newline at end of file
