@@ -4,6 +4,7 @@ import ee
 import math
 import copy
 import datetime
+from pipetools import pipe
 from pprint import pformat
 from functools import partial
 from ee.ee_exception import EEException
@@ -94,9 +95,10 @@ class Dataset:
             try:
                 imgcollection = imgcollection.map(self.qa)
             except AttributeError:
-                raise AttributeError("qa() method is not defined...please define one or set `use_qa` to False")
-        
-        
+                raise AttributeError(
+                    "qa() method is not defined...please define one or set `use_qa` to False"
+                )
+
         self.collection = imgcollection
 
     def __repr__(self):
@@ -152,8 +154,58 @@ class Dataset:
         )
         return eeDates.getInfo()
 
+    @staticmethod
+    def from_imgcollection(img_collection):
+        """Static method to convert an ee.ImageCollection object to a hf.Dataset object.
+        This method will take some time as it uses computed ee Objects from the image collection
+        propeties to populate the Dataset object properties (passing info from server to client)
+
+        args:
+            img_collection (ee.ImageCollection): computed ee.ImageCollection object to create a hf.Dataset
+
+        returns:
+            hf.Dataset: dataset object with property information directly from the ee.ImageCollection
+        """
+
+        # get region and date information
+        region = (
+            img_collection.map(geeutils.get_geoms)
+            .union(maxError=100)
+            .geometry(maxError=100)
+        )
+        # convert ee.Date info to string format
+        dates = (
+            img_collection.aggregate_array("system:time_start")
+            .sort()
+            .map(lambda x: ee.Date(x).format("YYYY-MM-dd HH:mm:ss.S"))
+        )
+
+        # pull the date info to local strings
+        start_time = dates.get(0).getInfo()
+        end_time = dates.get(-1).getInfo()
+
+        # get the collection id for `.asset_id` property
+        collection_id = img_collection.get("system:id").getInfo()
+        if collection_id is "None":
+            collection_id = "Custom ImageCollection"
+
+        # make a dummy dataset
+        dummy_ds = Dataset(
+            region,
+            start_time,
+            end_time,
+            asset_id="NOAA/VIIRS/001/VNP09GA",
+            use_qa=False,
+        )
+
+        # override the dummy dataset information with the correct data fr
+        dummy_ds.asset_id = collection_id
+        dummy_ds.collection = img_collection
+
+        return dummy_ds
+
     def copy(self):
-        """Returns a deep copy of the hydrafloods dataset class
+        """returns a deep copy of the hydrafloods dataset class
         """
         return copy.deepcopy(self)
 
@@ -162,12 +214,12 @@ class Dataset:
         Makes a copy of the collection and reassigns the image collection propety.
         Function must accept an ee.ImageCollection and return an ee.ImageCollection
 
-        Args:
+        args:
             func (object): Function to map across image collection. Function must accept ee.Image as first argument
             inplace (bool, optional): define whether to return another dataset object or update inplace. default = False
             **kwargs: arbitrary keyword to pass to `func`
 
-        Returns:
+        returns:
             Dataset | None: copy of class with results from `func` as image within collection property
         """
 
@@ -290,7 +342,7 @@ class Dataset:
             return outCls
 
     def aggregate_time(
-        self, dates=None, period=1, reducer="mean", clip_to_area=False, inplace=False
+        self, dates=None, period=1, period_unit="day", reducer="mean", clip_to_area=False, inplace=False
     ):
         """Aggregates multiple images into one based on time periods and a user defined reducer.
         Useful for mosaicing images from same date or time period.
@@ -300,6 +352,7 @@ class Dataset:
             dates (list[str], optional): list of dates defined as beginning time period of aggregatation. default = None,
                 all available uniques dates in collection will be used
             period (int, optional): number of days to advance from dates for aggregation. default = 1
+            period_unit (str, optional): time unit to advance period for aggregation. default = "day"
             reducer (str | ee.Reducer, optional): reducer to apply to images for aggregation, accepts string reducer name 
                 or ee.Reducer opbject, default = "mean"
             clip_to_area (bool): switch to clip imagery that has been merged to the overlaping region of imagery, default=False
@@ -314,7 +367,7 @@ class Dataset:
             """Closure function to map through days and reduce data within a given time period
             """
             t1 = ee.Date(d)
-            t2 = t1.advance(period, "day")
+            t2 = t1.advance(period, period_unit)
             img = (
                 self.collection.filterDate(t1, t2)
                 .reduce(reducer)
@@ -369,6 +422,77 @@ class Dataset:
             .set("system:time_start", img.get("system:time_start"))
         )
 
+    def pipe(self, steps, inplace=False,carry_metadata=True):
+        """Method to pipe imagery within dataset through multiple functions at once.
+        Assumes the first argument into piped functions are and ee.Image
+
+        args:
+            steps (list | tuple): iterable of functions/steps to apply to imagery.
+                list must be in the form of (func,func) or with a tuple of function/keyword ((func,kwargs),func)
+            inplace (bool, optional): define whether to return another dataset object or update inplace. default = False
+
+        returns:
+            Dataset | None: returns dataset.collection with piped functions applied
+
+        Example:
+            ```python
+            s1 = hf.Sentinel1(ee.Geometry.Point(105.03,11.72),"2019-10-03","2019-10-05")
+            water = s1.pipe(
+                (
+                    hf.gamma_map, #apply speckle filter
+                    (hf.egde_ostu,{'initial_threshold:-16}) # apply water mapping
+                )
+            )
+            ```
+        """
+
+        def _piper(funcs):
+            """Closure function to nest list of functions
+            """
+            if len(funcs) > 1:
+                one_shotter = funcs[0]
+                for func in funcs[1:]:
+                    one_shotter = pipe | one_shotter | func
+
+            else:
+                one_shotter = funcs[0]
+
+            return one_shotter
+
+        fs = []
+        # loop through the steps and create partial funcs is kwargs are provided
+        for step in steps:
+            try:
+                func, kwargs = step
+
+            except TypeError:
+                func = step
+                kwargs = None
+
+            if kwargs is not None:
+                pfunc = partial(func, **kwargs)
+            else:
+                pfunc = func
+
+            fs.append(pfunc)
+
+        # get the piped function
+        if carry_metadata:
+            one_shot = decorators.carry_metadata(_piper(fs))
+        else:
+            one_shot = _piper(fs)
+
+        # apply pipe to each image
+        out_coll = self.collection.map(lambda img: one_shot(img))
+
+        if inplace:
+            self.collection = out_coll
+            return
+        else:
+            outCls = self.copy()
+            outCls.collection = out_coll
+            return outCls
+
 
 class Sentinel1(Dataset):
     """Class extending dataset for the Sentinel 1 collection
@@ -383,7 +507,9 @@ class Sentinel1(Dataset):
             use_qa (bool, optional): boolean to determine to use a private `self.qa()` function. default=True
             **kwargs (optional): addtional arbitrary keywords to pass to `Dataset`
         """
-        super(Sentinel1, self).__init__(*args, asset_id=asset_id, use_qa=use_qa, **kwargs)
+        super(Sentinel1, self).__init__(
+            *args, asset_id=asset_id, use_qa=use_qa, **kwargs
+        )
 
         self.collection = self.collection.filter(
             ee.Filter.listContains("transmitterReceiverPolarisation", "VH")
@@ -398,16 +524,18 @@ class Sentinel1(Dataset):
         angles = img.select("angle")
         return img.updateMask(angles.lt(45).And(angles.gt(30)))
 
-    def add_fusion_features(self, inplace=False):
-        """Method to add additional features to SAR imagery for data fusion.
-        Will calculate Normalized Difference Polorization Index (VV-VH)/(VV+VH),
-        VV/VH ratio, and categorical orbit bands
+    def add_orbit_band(self, inplace=False):
+        """Method to add orbit band from S1 image metadata
+        Useful for determining if pixels are from ascending or descending orbits
+
+        args:
+            inplace (bool, optional): define whether to return another dataset object or update inplace. default = False
 
         returns:
             Dataset | None: returns dataset.collection where imagery has the added bands
         """
 
-        def _add_fusion_features(img):
+        def _add_features(img):
             """Closure function to add features as bands to the images
             """
             bounds = img.geometry(100)
@@ -415,18 +543,12 @@ class Sentinel1(Dataset):
             orbit_band = ee.Algorithms.If(
                 orbit.compareTo("DESCENDING"), ee.Image(1), ee.Image(0)
             )
-            vv = img.select("VV")
-            vh = img.select("VH")
-            ratio = vv.divide(vh).rename("ratio")
-            ndpi = vv.subtract(vh).divide(vv.add(vh)).rename("ndpi")
 
-            extraFeatures = ee.Image.cat(
-                [ee.Image(orbit_band).rename("orbit"), ratio, ndpi]
-            )
+            extraFeatures = ee.Image(orbit_band).rename("orbit")
 
             return img.addBands(extraFeatures.clip(bounds))
 
-        return self.apply_func(_add_fusion_features, inplace=inplace)
+        return self.apply_func(_add_features, inplace=inplace)
 
 
 class Viirs(Dataset):
@@ -434,8 +556,9 @@ class Viirs(Dataset):
         self,
         *args,
         asset_id="NOAA/VIIRS/001/VNP09GA",
-        apply_band_adjustment=False,
         use_qa=True,
+        apply_band_adjustment=False,
+        rescale=False,
         **kwargs,
     ):
         """Initialize VIIRS Dataset class
@@ -443,8 +566,9 @@ class Viirs(Dataset):
         args:
             *args: positional arguments to pass to `Dataset` (i.e. `region`, `start_time`, `end_time`)
             asset_id (str): asset id of the VIIRS earth engine collection. default="NOAA/VIIRS/001/VNP09GA"
-            apply_band_adjustment (bool, optional): boolean switch to apply linear band pass equation to convert values to Landsat8. default=False
             use_qa (bool, optional): boolean to determine to use a private `self.qa()` function. default=True
+            apply_band_adjustment (bool, optional): boolean switch to apply linear band pass equation to convert values to Landsat8. default=False
+            rescale (bool, optional): boolean switch to convert units from scaled int (0-10000) to float (0-1). If false values will be scaled int. default = False
             **kwargs (optional): addtional arbitrary keywords to pass to `Dataset`
         """
         super(Viirs, self).__init__(*args, asset_id=asset_id, use_qa=use_qa, **kwargs)
@@ -466,6 +590,9 @@ class Viirs(Dataset):
             ).multiply(10000)
             coll = coll.map(self.band_pass_adjustment)
 
+        if rescale:
+            coll = coll.map(geeutils.rescale)
+
         self.collection = coll
 
         return
@@ -488,7 +615,9 @@ class Viirs(Dataset):
 
 
 class Modis(Dataset):
-    def __init__(self, *args, asset_id="MODIS/006/MOD09GA", use_qa=True, **kwargs):
+    def __init__(
+        self, *args, asset_id="MODIS/006/MOD09GA", use_qa=True, rescale=False, **kwargs
+    ):
         """Initialize MODIS Dataset class
         Can be used with MOD09GA and MYD09GA
 
@@ -496,6 +625,7 @@ class Modis(Dataset):
             *args: positional arguments to pass to `Dataset` (i.e. `region`, `start_time`, `end_time`)
             asset_id (str): asset id of the MODIS earth engine collection. default="MODIS/006/MOD09GA"
             use_qa (bool, optional): boolean to determine to use a private `self.qa()` function. default=True
+            rescale (bool, optional): boolean switch to convert units from scaled int (0-10000) to float (0-1). If false values will be scaled int. default = False
             **kwargs (optional): addtional arbitrary keywords to pass to `Dataset`
         """
         super(Modis, self).__init__(*args, asset_id=asset_id, use_qa=use_qa, **kwargs)
@@ -504,6 +634,9 @@ class Modis(Dataset):
             self.BANDREMAP.get("modis"), self.BANDREMAP.get("new")
         )
 
+        if rescale:
+            self.collection = self.collection.map(geeutils.rescale)
+
         self.clip_to_region(inplace=True)
 
         return
@@ -522,7 +655,14 @@ class Modis(Dataset):
 
 
 class Landsat8(Dataset):
-    def __init__(self, *args, asset_id="LANDSAT/LC08/C01/T1_SR", use_qa=True, **kwargs):
+    def __init__(
+        self,
+        *args,
+        asset_id="LANDSAT/LC08/C01/T1_SR",
+        use_qa=True,
+        rescale=False,
+        **kwargs,
+    ):
         """Initialize Landsat8 Dataset class
         Can theoretically be useds with any Landsat surface reflectance collection (e.g. LANDSAT/LT05/C01/T1_SR)
 
@@ -530,14 +670,20 @@ class Landsat8(Dataset):
             *args: positional arguments to pass to `Dataset` (i.e. `region`, `start_time`, `end_time`)
             asset_id (str): asset id of the Landsat earth engine collection. default="LANDSAT/LC08/C01/T1_SR"
             use_qa (bool, optional): boolean to determine to use a private `self.qa()` function. default=True
+            rescale (bool, optional): boolean switch to convert units from scaled int (0-10000) to float (0-1). If false values will be scaled int. default = False
             **kwargs (optional): addtional arbitrary keywords to pass to `Dataset`
         """
-        super(Landsat8, self).__init__(*args, asset_id=asset_id, use_qa=use_qa, **kwargs)
+        super(Landsat8, self).__init__(
+            *args, asset_id=asset_id, use_qa=use_qa, **kwargs
+        )
 
         self.collection = self.collection.select(
             self.BANDREMAP.get("landsat8"), self.BANDREMAP.get("new")
         )
 
+        if rescale:
+            self.collection = self.collection.map(geeutils.rescale)
+
         return
 
     @decorators.carry_metadata
@@ -557,8 +703,9 @@ class Landsat7(Dataset):
         self,
         *args,
         asset_id="LANDSAT/LE07/C01/T1_SR",
-        apply_band_adjustment=False,
         use_qa=True,
+        apply_band_adjustment=False,
+        rescale=False,
         **kwargs,
     ):
         """Initialize Landsat7 Dataset class
@@ -567,11 +714,14 @@ class Landsat7(Dataset):
         args:
             *args: positional arguments to pass to `Dataset` (i.e. `region`, `start_time`, `end_time`)
             asset_id (str): asset id of the Landsat7 earth engine collection. default="LANDSAT/LE07/C01/T1_SR"
-            apply_band_adjustment (bool, optional): boolean switch to apply linear band pass equation to convert values to Landsat8. default=False
             use_qa (bool, optional): boolean to determine to use a private `self.qa()` function. default=True
+            apply_band_adjustment (bool, optional): boolean switch to apply linear band pass equation to convert values to Landsat8. default=False
+            rescale (bool, optional): boolean switch to convert units from scaled int (0-10000) to float (0-1). If false values will be scaled int. default = False
             **kwargs (optional): addtional arbitrary keywords to pass to `Dataset`
         """
-        super(Landsat7, self).__init__(*args, asset_id=asset_id, use_qa=use_qa, **kwargs)
+        super(Landsat7, self).__init__(
+            *args, asset_id=asset_id, use_qa=use_qa, **kwargs
+        )
 
         coll = self.collection.select(
             self.BANDREMAP.get("landsat7"), self.BANDREMAP.get("new")
@@ -589,6 +739,9 @@ class Landsat7(Dataset):
             ).multiply(10000)
             coll = coll.map(self.band_pass_adjustment)
 
+        if rescale:
+            coll = coll.map(geeutils.rescale)
+
         self.collection = coll
 
         return
@@ -610,8 +763,9 @@ class Sentinel2(Dataset):
         self,
         *args,
         asset_id="COPERNICUS/S2_SR",
-        apply_band_adjustment=False,
         use_qa=True,
+        apply_band_adjustment=False,
+        rescale=False,
         **kwargs,
     ):
         """Initialize Sentinel2 Dataset class
@@ -619,11 +773,14 @@ class Sentinel2(Dataset):
         args:
             *args: positional arguments to pass to `Dataset` (i.e. `region`, `start_time`, `end_time`)
             asset_id (str): asset id of the Sentinel2 earth engine collection. default="COPERNICUS/S2_SR"
-            apply_band_adjustment (bool, optional): boolean switch to apply linear band pass equation to convert values to Landsat8. default=False
             use_qa (bool, optional): boolean to determine to use a private `self.qa()` function. default=True
+            apply_band_adjustment (bool, optional): boolean switch to apply linear band pass equation to convert values to Landsat8. default=False
+            rescale (bool, optional): boolean switch to convert units from scaled int (0-10000) to float (0-1). If false values will be scaled int. default = False
             **kwargs (optional): addtional arbitrary keywords to pass to `Dataset`
         """
-        super(Sentinel2, self).__init__(*args, asset_id=asset_id, use_qa=use_qa, **kwargs)
+        super(Sentinel2, self).__init__(
+            *args, asset_id=asset_id, use_qa=use_qa, **kwargs
+        )
 
         coll = self.collection.select(
             self.BANDREMAP.get("sen2"), self.BANDREMAP.get("new")
@@ -641,6 +798,9 @@ class Sentinel2(Dataset):
             ).multiply(10000)
             coll = coll.map(self.band_pass_adjustment)
 
+        if rescale:
+            coll = coll.map(geeutils.rescale)
+
         self.collection = coll
 
         return
@@ -649,6 +809,56 @@ class Sentinel2(Dataset):
     def qa(self, img):
         """Custom QA masking method for Sentinel2 surface reflectance dataset
         """
-        sclImg = img.select("SCL")  # Scene Classification Map
-        mask = sclImg.gte(4).And(sclImg.lte(6))
-        return img.updateMask(mask)
+        CLD_PRB_THRESH = 40
+        NIR_DRK_THRESH = 0.175 * 1e4
+        CLD_PRJ_DIST = 3
+        BUFFER = 100
+        CRS = img.select(0).projection()
+
+        # Get s2cloudless image, subset the probability band.
+        cld_prb = ee.Image(
+            ee.ImageCollection("COPERNICUS/S2_CLOUD_PROBABILITY")
+            .filter(ee.Filter.eq("system:index", img.get("system:index")))
+            .first()
+        ).select("probability")
+
+        # Condition s2cloudless by the probability threshold value.
+        is_cloud = cld_prb.gt(CLD_PRB_THRESH)
+
+        # Identify water pixels from the SCL band, invert.
+        not_water = img.select("SCL").neq(6)
+
+        # Identify dark NIR pixels that are not water (potential cloud shadow pixels).
+        dark_pixels = img.select("B8").lt(NIR_DRK_THRESH).multiply(not_water)
+
+        # Determine the direction to project cloud shadow from clouds (assumes UTM projection).
+        shadow_azimuth = ee.Number(90).subtract(
+            ee.Number(img.get("MEAN_SOLAR_AZIMUTH_ANGLE"))
+        )
+
+        # Project shadows from clouds for the distance specified by the CLD_PRJ_DIST input.
+        cld_proj = (
+            is_cloud.directionalDistanceTransform(shadow_azimuth, CLD_PRJ_DIST * 10)
+            .reproject(**{"crs": CRS, "scale": 120})
+            .select("distance")
+            .mask()
+        )
+
+        # Identify the intersection of dark pixels with cloud shadow projection.
+        is_shadow = cld_proj.multiply(dark_pixels)
+
+        # Combine cloud and shadow mask, set cloud and shadow as value 1, else 0.
+        is_cld_shdw = is_cloud.add(is_shadow).gt(0)
+
+        # Remove small cloud-shadow patches and dilate remaining pixels by BUFFER input.
+        # 20 m scale is for speed, and assumes clouds don't require 10 m precision.
+        is_cld_shdw = (
+            is_cld_shdw.focal_min(2)
+            .focal_max(BUFFER * 2 / 20)
+            .reproject(**{"crs": CRS, "scale": 60})
+            .rename("cloudmask")
+        )
+
+        # Subset reflectance bands and update their masks, return the result.
+        return img.select("B.*").updateMask(is_cld_shdw.Not())
+
