@@ -21,9 +21,14 @@ def lee_sigma(img, window=9, sigma=0.9, looks=4, tk=7, keep_bands="angle"):
         ee.Image: filtered SAR image using the Lee Sigma algorithm
     """
     band_names = img.bandNames()
-    proc_bands = band_names.remove(keep_bands)
-    keep_img = img.select(keep_bands)
+    if keep_bands is not None:
+        keep_img = img.select(keep_bands)
+        proc_bands = band_names.remove(keep_bands)
+    else:
+        proc_bands = band_names
+
     img = img.select(proc_bands)
+    
 
     midPt = (window // 2) + 1 if (window % 2) != 0 else window // 2
     kernelWeights = ee.List.repeat(ee.List.repeat(1, window), window)
@@ -115,7 +120,12 @@ def lee_sigma(img, window=9, sigma=0.9, looks=4, tk=7, keep_bands="angle"):
     retainPixel = K.gte(tk)
     xHat = geeutils.power_to_db(img.updateMask(retainPixel).unmask(mmse))
 
-    return ee.Image(xHat).rename(proc_bands).addBands(keep_img)
+    output = ee.Image(xHat).rename(proc_bands)
+
+    if keep_bands is not None:
+        output = output.addBands(keep_img)
+
+    return output
 
 
 # The RL speckle filter
@@ -314,9 +324,10 @@ def refined_lee(image):
 
     bandNames = image.bandNames()
     power = geeutils.db_to_power(image)
+    bandList = ee.List.sequence(0,bandNames.length().subtract(1))
 
-    result = ee.ImageCollection(bandNames.map(apply_filter)).toBands().rename(bandNames)
-    return geeutils.power_to_db(ee.Image(result))
+    result = ee.ImageCollection(bandList.map(apply_filter)).toBands()
+    return geeutils.power_to_db(ee.Image(result)).rename(bandNames)
 
 
 @decorators.carry_metadata
@@ -369,7 +380,9 @@ def gamma_map(img, window=7, enl=4.9):
     )
     f = b.multiply(mean).add(d.sqrt()).divide(alpha.multiply(2.0))
 
-    caster = ee.Dictionary.fromLists(bandNames, ee.List.repeat("float", 3))
+    caster = ee.Dictionary.fromLists(
+        bandNames, ee.List.repeat("float", bandNames.length())
+    )
     img1 = (
         geeutils.power_to_db(mean.updateMask(ci.lte(cu))).rename(bandNames).cast(caster)
     )
@@ -405,6 +418,13 @@ def p_median(img, window=5):
         ee.Image: filtered image
     """
 
+    def _band_filter(bname):
+        selector = ee.List([bname])
+        band_img = img.select(selector)
+        hv_median = band_img.reduceNeighborhood(ee.Reducer.median(), hv_kernel)
+        diag_median = band_img.reduceNeighborhood(ee.Reducer.median(), diag_kernel)
+        return ee.Image(ee.Image.cat([hv_median, diag_median]).reduce("mean")).rename(selector)
+
     if window % 2 == 0:
         window += 1
 
@@ -427,9 +447,85 @@ def p_median(img, window=5):
     hv_kernel = ee.Kernel.fixed(window, window, hv_weights)
     diag_kernel = ee.Kernel.fixed(window, window, diag_weights)
 
-    hv_median = img.reduceNeighborhood(ee.Reducer.median(), hv_kernel)
+    reduced_bands = ee.ImageCollection.fromImages(
+        band_names.map(_band_filter)
+    ).toBands()
 
-    diag_median = img.reduceNeighborhood(ee.Reducer.median(), diag_kernel)
+    return reduced_bands.rename(band_names)
 
-    return ee.Image.cat([hv_median, diag_median]).reduce("mean").rename(band_names)
 
+@decorators.carry_metadata
+def perona_malik(img, n_iters=10, K=3, method=1):
+    """	Perona-Malik (anisotropic diffusion) convolution
+    Developed by Gennadii Donchyts see https://groups.google.com/g/google-earth-engine-developers/c/umGlt5qIN1I/m/PD8lsJ7qBAAJ
+    I(n+1, i, j) = I(n, i, j) + lambda * (cN * dN(I) + cS * dS(I) + cE * dE(I), cW * dW(I))
+    
+    args:
+        img (ee.Image): Earth engine image object. Expects that imagery is a SAR image
+        n_iters (int, optional): Number of interations to apply filter
+	    K (int,optional): moving window size to apply filter (i.e. a value of 7 == 7x7 window). default = 3
+        method (int, optional): choose method 1 (default) or 2
+    returns:
+        ee.Image: filtered SAR image using the perona malik algorithm
+    """
+
+    def _method_1(dI_W, dI_E, dI_N, dI_S):
+        cW = dI_W.pow(2).multiply(k1).exp()
+        cE = dI_E.pow(2).multiply(k1).exp()
+        cN = dI_N.pow(2).multiply(k1).exp()
+        cS = dI_S.pow(2).multiply(k1).exp()
+
+        return cW, cE, cN, cS
+
+    def _method_2(dI_W, dI_E, dI_N, dI_S):
+        cW = one.divide(one.add(dI_W.pow(2).divide(k2)))
+        cE = one.divide(one.add(dI_E.pow(2).divide(k2)))
+        cN = one.divide(one.add(dI_N.pow(2).divide(k2)))
+        cS = one.divide(one.add(dI_S.pow(2).divide(k2)))
+
+        return cW, cE, cN, cS
+
+    # covnert db to natural units  before applying filter
+    power = geeutils.db_to_power(img)
+
+    dxW = ee.Kernel.fixed(3, 3, [[0, 0, 0], [1, -1, 0], [0, 0, 0]])
+    dxE = ee.Kernel.fixed(3, 3, [[0, 0, 0], [0, -1, 1], [0, 0, 0]])
+    dyN = ee.Kernel.fixed(3, 3, [[0, 1, 0], [0, -1, 0], [0, 0, 0]])
+    dyS = ee.Kernel.fixed(3, 3, [[0, 0, 0], [0, -1, 0], [0, 1, 0]])
+
+    one = ee.Image.constant(1.0)
+    l = ee.Image.constant(0.2)
+    k = ee.Image.constant(K)
+    k1 = ee.Image.constant(-1.0 / K)
+    k2 = k.pow(2)
+
+    if method == 1:
+        _method = _method_1
+    elif method == 2:
+        _method = _method_2
+    else:
+        raise NotImplementedError(
+            "Could not determine algorithm to apply filter...options for `method` are 1 or 2"
+        )
+
+    for i in range(n_iters):
+        dI_W = power.convolve(dxW)
+        dI_E = power.convolve(dxE)
+        dI_N = power.convolve(dyN)
+        dI_S = power.convolve(dyS)
+
+        cW, cE, cN, cS = _method(dI_W, dI_E, dI_N, dI_S)
+
+        power = power.add(
+            l.multiply(
+                cN.multiply(dI_N)
+                .add(cS.multiply(dI_S))
+                .add(cE.multiply(dI_E))
+                .add(cW.multiply(dI_W))
+            )
+        )
+
+    # covnert natural to db units after filter is done
+    img = geeutils.power_to_db(power)
+
+    return img
