@@ -5,6 +5,7 @@ import string
 import random
 import datetime
 from hydrafloods import decorators
+import hydrafloods.indices as all_indices
 
 
 # helper function to convert qa bit image to flag
@@ -180,6 +181,21 @@ def batch_export(
     return
 
 
+@decorators.carry_metadata
+def rescale(img, scale=0.0001, offset=0):
+    """Function to linearly rescale units using user defined scale and offset
+
+    args:
+        img (ee.Image): image to rescale
+        scale (float,optional): scale value (i.e. slope of linear equation). default = 0.0001
+        offset (float, optional): offset value (i.e. y-intercept). default = 0
+
+    returns:
+        ee.Image: rescaled image
+    """
+    return img.multiply(scale).add(offset)
+
+
 @decorators.carry_metadata
 def power_to_db(img):
     """Function to convert SAR units from power to dB
@@ -206,160 +222,6 @@ def db_to_power(img):
     return ee.Image(10).pow(img.divide(10))
 
 
-@decorators.carry_metadata
-def ndvi(img):
-    """Function to calculate Normalized Difference Vegetation Index (NDVI).
-    Expects image has "nir" and "red" bands.
-
-    args:
-        img (ee.Image): image to calculate NDVI
-
-    returns:
-        ee.Image: NDVI image
-    """
-    return img.normalizedDifference(["nir", "red"]).rename("ndvi")
-
-
-@decorators.carry_metadata
-def evi(img):
-    """Function to calculate Enhanced Vegetation Index (EVI).
-    Expects image has "blue", "red", and "nir" bands.
-
-    args:
-        img (ee.Image): image to calculate EVI
-
-    returns:
-        ee.Image: EVI image
-    """
-    return img.expression(
-        "2.5*(nir-red)/(nir+6.0*red-7.5*blue+1)",
-        {
-            "blue": img.select("blue"),
-            "red": img.select("red"),
-            "nir": img.select("nir"),
-        },
-    ).rename("evi")
-
-
-@decorators.carry_metadata
-def mndwi(img):
-    """Function to calculate modified Difference Water Index (MNDWI).
-    Expects image has "green" and "swir1" bands.
-
-    args:
-        img (ee.Image): image to calculate MNDWI
-
-    returns:
-        ee.Image: MNDWI image
-    """
-    return img.normalizedDifference(["green", "swir1"]).rename("mndwi")
-
-
-@decorators.carry_metadata
-def nwi(img):
-    """Function to calculate new water index (NWI).
-    Expects image has "blue", "nir", "swir1" and "swir2" bands.
-
-    args:
-        img (ee.Image): image to calculate NWI
-
-    returns:
-        ee.Image: NWI image
-    """
-    return img.expression(
-        "((b-(n+s+w))/(b+(n+s+w))*100)",
-        {
-            "b": img.select("blue"),
-            "n": img.select("nir"),
-            "s": img.select("swir1"),
-            "w": img.select("swir2"),
-        },
-    ).rename("nwi")
-
-
-@decorators.carry_metadata
-def gwi(img):
-    """Function to calculate general water index (GWI)
-    Expects image has "green", "red", "nir", and "swir1" bands.
-
-    args:
-        img (ee.Image): image to calculate GWI
-
-    returns:
-        ee.Image: GWI image
-    """
-    return img.expression(
-        "(g+r)-(n+s)",
-        {
-            "g": img.select("green"),
-            "r": img.select("red"),
-            "n": img.select("nir"),
-            "s": img.select("swir1"),
-        },
-    ).rename("gwi")
-
-
-@decorators.carry_metadata
-def aewinsh(img):
-    """Function to calculate automated water extraction index (AEWI) no shadow
-    Expects image has "green", "nir", "swir1" and "swir2" bands.
-
-    args:
-        img (ee.Image): image to calculate AEWInsh
-
-    returns:
-        ee.Image:  AEWInsh image
-    """
-    return img.expression(
-        "4.0 * (g-s) - ((0.25*n) + (2.75*w))",
-        {
-            "g": img.select("green"),
-            "s": img.select("swir1"),
-            "n": img.select("nir"),
-            "w": img.select("swir2"),
-        },
-    ).rename("aewinsh")
-
-
-@decorators.carry_metadata
-def aewish(img):
-    """Function to calculate automated water extraction index (AEWI) shadow
-    Expects image has "blue", "green", "nir", "swir1" and "swir2" bands.
-
-    args:
-        img (ee.Image): image to calculate AEWIsh
-
-    returns:
-        ee.Image:  AEWIsh image
-    """
-    return img.expression(
-        "b+2.5*g-1.5*(n+s)-0.25*w",
-        {
-            "b": img.select("blue"),
-            "g": img.select("green"),
-            "n": img.select("nir"),
-            "s": img.select("swir1"),
-            "w": img.select("swir2"),
-        },
-    ).rename("aewish")
-
-
-@decorators.carry_metadata
-def lswi(img):
-    """Function to calculate land surface water index (LSWI).
-    Expects image has "nir" and "swir1" bands.
-
-    args:
-        img (ee.Image): image to calculate LSWI
-
-    returns:
-        ee.Image: LSWI image
-    """
-    return img.expression(
-        "(nir-swir)/(nir+swir)", {"nir": img.select("nir"), "swir": img.select("swir1")}
-    ).rename("lswi")
-
-
 @decorators.carry_metadata
 def add_indices(img, indices=["mndwi"]):
     """Function to calculate multiple band indices and add to image as bands
@@ -367,20 +229,17 @@ def add_indices(img, indices=["mndwi"]):
     args:
         img (ee.Image): image to calculate indices from
         indices (list[str], optional): list of strings of index names to calculate. 
-            can use any named index function in geeutils. default = ["ndvi"]
+            can use any named index function in geeutils. default = ["mndwi"]
 
     returns:
         ee.Image: image object with added indices
     """
-    # create a dict to look up index functions
-
-    local_funcs = globals()
 
     # loop through each index and append to images list
     cat_bands = [img]
     for index in indices:
-
-        cat_bands.append(local_funcs[index](img))
+        index_func = getattr(all_indices, index)
+        cat_bands.append(index_func(img))
 
     # return images as concatenated bands
     return ee.Image.cat(cat_bands)
@@ -409,26 +268,26 @@ def tile_region(region, grid_size=0.1, intersect_geom=None, contain_geom=None):
             j = ee.Number(j)
             box = ee.Feature(
                 ee.Geometry.Rectangle(
-                    [j, i, j.add(grid_size), i.add(grid_size)],
+                    [j, i, j.add(grid_res), i.add(grid_res)],
                     "epsg:4326",
                     geodesic=False,
                 )
             )
             if contain_geom is not None:
                 out = ee.Algorithms.If(
-                    region.contains(contain_geom, maxError=10), box, None
+                    box.contains(contain_geom, maxError=1000), box, None
                 )
             elif intersect_geom is not None:
                 out = ee.Algorithms.If(
-                    region.intersects(intersect_geom, maxError=10), box, None
+                    box.intersects(intersect_geom, maxError=1000), box, None
                 )
             else:
                 out = box
             return ee.Feature(out)
 
         i = ee.Number(i)
-        out = ee.List.sequence(west, east.subtract(grid_size), grid_size).map(
-            contructXGrid
+        out = ee.List.sequence(west, east.subtract(grid_res), grid_res).map(
+            contructXGrid, True
         )
         return out
 
@@ -437,7 +296,7 @@ def tile_region(region, grid_size=0.1, intersect_geom=None, contain_geom=None):
             "contains and intersection keywords are mutually exclusive, please define only one"
         )
 
-    bounds = region.bounds(maxError=100)
+    bounds = region.bounds(maxError=1000)
     coords = ee.List(bounds.coordinates().get(0))
     grid_res = ee.Number(grid_size)
 
@@ -446,8 +305,16 @@ def tile_region(region, grid_size=0.1, intersect_geom=None, contain_geom=None):
     east = ee.Number(ee.List(coords.get(2)).get(0))
     north = ee.Number(ee.List(coords.get(2)).get(1))
 
-    west = west.subtract(west.mod(grid_res))
-    south = south.subtract(south.mod(grid_res))
+    west = ee.Algorithms.If(
+        west.lt(0),
+        west.subtract(west.mod(grid_res).add(grid_res)),
+        west.subtract(west.mod(grid_res)),
+    )
+    south = ee.Algorithms.If(
+        south.lt(0),
+        south.subtract(south.mod(grid_res).add(grid_res)),
+        south.subtract(south.mod(grid_res)),
+    )
     east = east.add(grid_res.subtract(east.mod(grid_res)))
     north = north.add(grid_res.subtract(north.mod(grid_res)))
 
@@ -460,7 +327,7 @@ def tile_region(region, grid_size=0.1, intersect_geom=None, contain_geom=None):
     return grid
 
 
-def country_bbox(country_name, max_error=100):
+def country_bbox(country_name, max_error=1000):
     """Function to get a bounding box geometry of a country
 
     args:
