@@ -1,11 +1,54 @@
 import ee
+from collections import OrderedDict
 from hydrafloods import decorators
 
+@decorators.carry_metadata
+def apply_fcnn(image, project_name, model_name, model_kwargs=None,output_probas=False,output_names=None):
+    """
+    args:
+        output_bands (Iterable, optional): list of names to 
+    """
 
-def logistic_regression():
+    if model_kwargs is None:
+        model_kwargs = OrderedDict(
+            projectName = project_name,
+            modelName =  model_name
+        )
+    else:
+        positional = OrderedDict(
+            projectName = project_name,
+            modelName =  model_name
+        )
 
-    return
+        model_kwargs = OrderedDict({**positional,**model_kwargs})
 
+    # Load the trained model and use it for prediction.
+    model = ee.Model.fromAiPlatformPredictor(**model_kwargs)
+
+    # run the predictions
+    predictions = model.predictImage(image.toFloat().toArray())
+
+    if output_probas:
+        if output_names is None:
+            raise ValueError("please provide `output_names` when `ouput_probas` is set to True")
+
+        output = (
+            predictions
+            .arrayFlatten([output_names])
+        )
+
+    else:
+        output_names = "classification" if output_names is None else output_names
+        # find highest probability class
+        output = (
+            predictions
+            .arrayArgmax()
+            .arrayFlatten([[output_names]])
+        )
+
+    return output
+
+    
 
 def minmax_scaling_dict(fc, feature_names):
     """Function to calculate the minimum and maximum values of feautures in a collection
@@ -258,7 +301,7 @@ def random_forest_ee(
 
     else:
         raise ValueError(
-            "Could not determine scaling option. Options are ['minmax', 'standar', or None]"
+            "Could not determine scaling option. Options are ['minmax', 'standard', or None]"
         )
 
     classifier = (
@@ -269,8 +312,62 @@ def random_forest_ee(
 
     return classifier, scaling_dict
 
+def gradient_boosting_ee(
+    n_trees,
+    feature_collection,
+    feature_names,
+    label,
+    scaling=None,
+    mode="classification",
+    learning_rate=0.01,
+    loss="LeastAbsoluteDeviation"
+):
+    """Helper function to scale feature collection and train gradient tree boosting model
 
-def pca(image, region=None, scale=90, max_pixels=1e9):
+    args:
+        n_trees (int): number of trees for gradient boosting model
+        feature_collection (ee.FeatureCollection): features to train random forest model
+        feature_names (list[str]): names of feature columns to use in random forest model (x values)
+        label (str): name of feature column to fit random forest model (y value)
+        scaling (str | None, optional): name of scaling to apply before training. One of: "minmax", "standard", `None`.
+            default = `None`
+        mode (str, optional): The output mode of the random forest model. One of: "classification", "regression",
+            "probability". default = "classification"
+        learning_rate (float,optional): The shrinkage parameter in (0, 1] controls the learning rate of procedure. default = 0.01
+        loss (str, optional): Loss function to be optimized. default = "LeastAbsoluteDeviation"
+    """
+
+    if scaling == "minmax":
+        scaling_dict = minmax_scaling_dict(feature_collection, feature_names)
+        fc_norm = minmax_feature_scaling(
+            feature_collection, scaling_dict, feature_names
+        )
+
+    elif scaling == "standard":
+        scaling_dict = standard_scaling_dict(feature_collection, feature_names)
+        fc_norm = standard_feature_scaling(
+            feature_collection, scaling_dict, feature_names
+        )
+
+    elif scaling is None:
+        scaling_dict = None
+        fc_norm = feature_collection
+
+    else:
+        raise ValueError(
+            "Could not determine scaling option. Options are ['minmax', 'standard', or None]"
+        )
+
+    classifier = (
+        ee.Classifier.smileGradientTreeBoost(numberOfTrees=n_trees,samplingRate=learning_rate,loss=loss)
+        .setOutputMode(mode.upper())
+        .train(fc_norm, label, feature_names)
+    )
+
+    return classifier, scaling_dict
+
+
+def calc_image_pca(image, region=None, scale=90, max_pixels=1e9):
     """Principal component analysis decomposition of image bands
 
     args:
@@ -281,13 +378,13 @@ def pca(image, region=None, scale=90, max_pixels=1e9):
         max_pixels (int, optional): maximum number of pixels to use in reduction operations. default = 1e9
 
     returns:
-        ee.Image: principal components scaled by 
+        ee.Image: principal components scaled by eigen values
     """
 
     bandNames = image.bandNames()
 
     out_band_names = ee.List.sequence(1, bandNames.length()).map(
-        lambda x: ee.String("pc_").cat(ee.Number(b).int())
+        lambda x: ee.String("pc_").cat(ee.Number(x).int())
     )
 
     # Mean center the data to enable a faster covariance reducer
@@ -342,3 +439,107 @@ def pca(image, region=None, scale=90, max_pixels=1e9):
         # Normalize the PCs by their SDs.
         .divide(sdImage)
     )
+
+
+def calc_feature_pca(fc,names,is_centered=False):
+    """Principal component decomposition of features
+
+    args:
+        fc (ee.FeatureCollection): feature collection to caluculate PCA from
+        names (list[str]): property names to uses as features in PCA
+        is_centered (bool, optional): boolean to identify if features need to be centered before PCA. 
+            False means apply centering. default = False
+
+    returns:
+        ee.Array: eigen vectors of PCA
+        ee.Array: eigen values of PCA
+        ee.Array: mean values of each feature
+    """
+    array_ = ee.Array(ee.List(fc.makeArray(names).aggregate_array("array").map(lambda x: ee.Array(x).toList())))
+    center = array_.reduce(ee.Reducer.mean(),[0]).repeat(0,array_.length().get([0]))
+    if not is_centered:
+        centered = array_.subtract(center)
+    else:
+        centered = array_
+    
+    # Compute the covariance of the bands within the region.
+    covar = centered.transpose().matrixMultiply(centered)
+    # Perform an eigen analysis and slice apart the values and vectors.
+    eigens = covar.eigen()
+    
+    out_band_names = [f"pc_{i}" for i in range(len(names))]
+    
+    eigen_vecs = eigens.slice(1, 1)
+    eigen_vals = eigens.slice(1, 0, 1)
+    
+    return eigen_vecs, eigen_vals, center.slice(0,0,1).project([1])
+
+
+def apply_feature_pca(fc, eigen_vecs, names, center=None):
+    """Applies Principal component decomposition on features
+
+    args:
+        fc (ee.FeatureCollection): feature collection to caluculate pricipal components from
+        eigen_vecs (ee.Array): eigen vectors of PCA to transform features
+        names (list[str]): property names to uses as features in PCA
+        center (ee.Array | None, optional): Array of mean values to center features. If None then no
+            centering is applies. default = None
+
+    returns:
+        ee.FeatureCollection: feacture collection with new properties within each feature being the principal components
+    """
+    array_ = ee.Array(ee.List(fc.makeArray(names).aggregate_array("array").map(lambda x: ee.Array(x).toList())))
+    if center is not None:
+        centered = array_.subtract(ee.Array.cat([center],1).transpose().repeat(0,array_.length().get([0])))
+    else:
+        centered = array_
+    
+    pca_arr = eigen_vecs.matrixMultiply(centered.transpose()).transpose()
+    
+    out_band_names = [f"pc_{i}" for i in range(len(names))]
+    
+    fc_size = fc.size()
+    fc_list = fc.toList(fc_size)
+    fc_pca = ee.FeatureCollection(
+        ee.List.sequence(0,fc_size.subtract(1)).map(
+            lambda x: ee.Feature(fc_list.get(x)).set(ee.Dictionary.fromLists(
+                out_band_names, pca_arr.slice(0,x,ee.Number(x).add(1),1).project([1]).toList()
+                )
+            )
+        )
+    )
+    
+    return fc_pca
+    
+def apply_image_pca(img, eigen_vecs, names, center=None):
+    """Applies Principal component decomposition on image
+
+    args:
+        img (ee.Image): image to caluculate pricipal components from
+        eigen_vecs (ee.Array): eigen vectors of PCA to transform features
+        names (list[str]): band names to uses as features in PCA
+        center (ee.Array | None, optional): Array of mean values to center features. If None then no
+            centering is applies. default = None
+
+    returns:
+        ee.Image: principal components calculated from image
+    """
+    if center is not None:
+        arrayImage = img.select(names).subtract(
+            ee.Image.constant(center.toList())
+        ).toArray().toArray(1)
+    else:
+        arrayImage = img.select(names).toArray().toArray(1)
+    
+    principalComponents = ee.Image(eigen_vecs).matrixMultiply(arrayImage)
+    
+    out_band_names = [f"pc_{i}" for i in range(len(names))]
+    
+    pcaImage = (principalComponents
+        # Throw out an an unneeded dimension, [[]] -> [].
+        .arrayProject([0])
+        # Make the one band array image a multi-band image, [] -> image.
+        .arrayFlatten([out_band_names])
+    )
+    
+    return pcaImage
\ No newline at end of file
