@@ -11,6 +11,7 @@ def bmax_otsu(
     region=None,
     scale=90,
     initial_threshold=0,
+    thresh_no_data=None,
     invert=False,
     grid_size=0.1,
     bmax_threshold=0.75,
@@ -30,6 +31,7 @@ def bmax_otsu(
         region (ee.Geometry | None, optional): region to determine threshold, if set to `None` will use img.geometry(). default = None
         scale (int, optional): scale at which to perform reduction operations, setting higher will prevent OOM errors. default = 90
         initial_threshold (float, optional): initial estimate of water/no-water for estimating the probabilities of classes in segment. default = 0
+        thresh_no_data (float, optional): threshold to be used when histogram is empty (likely little to no water present in image). default = -0.2
         invert (bool, optional): boolean switch to determine if to threshold greater than (True) or less than (False). default = False
         grid_size (float, optional): size in decimal degrees to tile image/region to check for bimodality. default = 0.1
         bmax_threshold (float, optional): value 0-1 to determine if a value of bmax is bimodal or not. default = 0.75
@@ -47,7 +49,7 @@ def bmax_otsu(
     def calcBmax(feature):
         """Closure function to calculate Bmax for each feature covering image
         """
-        segment = img 
+        segment = img
         initial = segment.lt(initial_threshold)
         p1 = ee.Number(
             initial.reduceRegion(
@@ -125,7 +127,18 @@ def bmax_otsu(
         tileScale=16,
     )
 
-    threshold = otsu(histogram.get(histBand.cat("_histogram")))
+    if thresh_no_data is not None:
+        threshold = ee.Number(
+            ee.Algorithms.If(
+                ee.Dictionary(histogram.get(histBand.cat("_histogram"))).contains(
+                    "bucketMeans"
+                ),
+                otsu(histogram.get(histBand.cat("_histogram"))),
+                thresh_no_data,
+            )
+        )
+    else:
+        threshold = otsu(histogram.get(histBand.cat("_histogram")))
 
     if return_threshold is True:
         return ee.Image(threshold)
@@ -141,12 +154,13 @@ def edge_otsu(
     region=None,
     scale=90,
     initial_threshold=0,
+    thresh_no_data=None,
     invert=False,
-    canny_threshold=0.05, 
-    canny_sigma=0, 
-    canny_lt=0.05,  
-    connected_pixels=200,  
-    edge_length=50, 
+    canny_threshold=0.05,
+    canny_sigma=0,
+    canny_lt=0.05,
+    connected_pixels=200,
+    edge_length=50,
     edge_buffer=100,
     max_buckets=255,
     min_bucket_width=0.001,
@@ -162,13 +176,14 @@ def edge_otsu(
         region (ee.Geometry | None, optional): region to determine threshold, if set to `None` will use img.geometry(). default = None
         scale (int, optional): scale at which to perform reduction operations, setting higher will prevent OOM errors. default = 90
         initial_threshold (float, optional): initial estimate of water/no-water for estimating the edges. default = 0
+        thresh_no_data (float, optional): threshold to be used when histogram is empty (likely little to no water present in image). default = -0.2
         invert (bool, optional): boolean switch to determine if to threshold greater than (True) or less than (False). default = False
         canny_threshold (float, optional): threshold for canny edge detection. default = 0.05
         canny_sigma (float, optional): sigma value for gaussian filter in canny edge detection. default = 0
         canny_lt (float, optional): lower threshold for canny detection. default = 0.05
         connected_pixels (int, optional): maximum size of the neighborhood in pixels to determine if connected. default = 200
         edge_length (int, optional): minimum length of edges from canny detection to be considered edge. default = 50
-        edge_buffer (int, optional): number of pixels to buffer edges on a side for histogram sampling. default = 100
+        edge_buffer (int, optional): distance in meters to buffer edges on a side for histogram sampling. default = 100
         max_buckets (int, optional): The maximum number of buckets to use when building a histogram; will be rounded up to a power of 2. default = 255
         min_bucket_width (float, optional): The minimum histogram bucket width to allow any power of 2. default = 0.001
         max_raw (int, optional): The number of values to accumulate before building the initial histogram. default = 1e6
@@ -213,7 +228,18 @@ def edge_otsu(
         tileScale=16,
     )
 
-    threshold = otsu(histogram.get(histBand.cat("_histogram")))
+    if thresh_no_data is not None:
+        threshold = ee.Number(
+            ee.Algorithms.If(
+                ee.Dictionary(histogram.get(histBand.cat("_histogram"))).contains(
+                    "bucketMeans"
+                ),
+                otsu(histogram.get(histBand.cat("_histogram"))),
+                thresh_no_data,
+            )
+        )
+    else:
+        threshold = otsu(histogram.get(histBand.cat("_histogram")))
 
     if return_threshold is True:
         return threshold
@@ -262,9 +288,19 @@ def otsu(histogram):
     return ee.Number(output)
 
 
-def kmeans_extent(img, hand, initial_threshold=0, region=None, band=None, scale=90):
+def kmeans_extent(
+    img,
+    hand,
+    initial_threshold=0,
+    region=None,
+    band=None,
+    n_samples=500,
+    seed=7,
+    invert=False,
+    scale=90,
+):
     """Water thresholding methodology using image values and HAND.
-    Method taken from https://doi.org/10.1016/j.rse.2020.111732
+    Method from https://doi.org/10.1016/j.rse.2020.111732
 
     args:
         img (ee.Image): input image to thresholding algorithm
@@ -272,12 +308,22 @@ def kmeans_extent(img, hand, initial_threshold=0, region=None, band=None, scale=
         initial_threshold (float, optional): initial estimate of water/no-water for stratified sampling. default = 0
         region (ee.Geometry | None, optional): region to sample values for KMeans clustering, if set to `None` will use img.geometry(). default = None
         band (str | None,optional): band name to use for thresholding, if set to `None` will use first band in image. default = None
+        samples (int, optional): number of stratified samples to gather for clustering from the initial water/no-water map. default=500
+        seed (int, optional): random number generator seed for sampling. default = 7
         scale (int, optional): scale at which to perform reduction operations, setting higher will prevent OOM errors. default = 90
 
     returns:
         ee.Image: clustered image from KMeans clusterer. Classes assumed to be water/no-water
 
     """
+
+    def _cluster_center(x):
+        return ee.Number(
+            (
+                band_arr.mask(classes.eq(ee.Number(x))).reduce(ee.Reducer.mean(), [0])
+            ).get([0])
+        )
+
     if region is None:
         region = img.geometry()
 
@@ -290,22 +336,142 @@ def kmeans_extent(img, hand, initial_threshold=0, region=None, band=None, scale=
 
     hand_band = ee.String(hand.bandNames().get(0))
 
-    strata = img.gt(initial_threshold).rename("strata")
+    # convert invert to a number 0 or 1 for ee server-side
+    invert = 1 if invert else 0
 
-    samples = ee.Image.cat([img, hand, strata]).stratifiedSample(
-        numPoints=50,
+    img = img.addBands(hand.unmask(0))
+
+    strata = img.select(band).gt(initial_threshold).rename("strata")
+
+    samples = img.addBands(strata).stratifiedSample(
+        numPoints=n_samples,
         classBand="strata",
         region=region,
         scale=scale,
         classValues=[0, 1],
-        classPoints=[500, 500],
+        classPoints=[n_samples, n_samples],
         tileScale=16,
-        seed=7,
+        seed=seed,
     )
 
-    clusterer = ee.Clusterer.wekaKMeans(2, 2).train(samples, [band, hand_band])
+    clusterer = ee.Clusterer.wekaKMeans(2, 1).train(samples, [band, hand_band])
 
-    water = ee.Image.cat([img, hand.unmask(0)]).cluster(clusterer)
+    samples = samples.cluster(clusterer, "classes")
+    classes = samples.aggregate_array("classes")
+    unique = classes.distinct().sort()
+    classes = ee.Array(classes)
+    band_arr = ee.Array(samples.aggregate_array(band))
+
+    class_means = unique.map(_cluster_center)
+
+    min_mean = class_means.reduce(ee.Reducer.min())
+
+    water_class = class_means.indexOf(min_mean)
+
+    water = img.cluster(clusterer)
+
+    water = ee.Image(ee.Algorithms.If(water_class.eq(0), water.Not(), water))
 
     return water.rename("water").uint8()
 
+
+def multidim_semisupervised(
+    img,
+    bands,
+    rank_band=None,
+    ranking="min",
+    region=None,
+    n_samples=500,
+    seed=7,
+    scale=90,
+    proba_threshold=None,
+):
+    """Implementation of the Automatic water detection from 
+    multidimensional hierarchical clustering algorithm.
+    Method details: https://doi.org/10.1016/j.rse.2020.112209
+    Note: this is a similar method, not exact from the paper
+
+    args:
+        img (ee.Image): input image to thresholding algorithm
+        bands (list | ee.List): band names to use for the semi supervised classification
+        rank_band (str, optional): band name used to rank which unserpervised class is water. If None then first band name in `bands` is used. default = None
+        ranking (str, optional): method to rank the classes by `rank_band`. Options are 'min' or 'max'. If 'min', then the lowest class mean is considered water. default = 'min'
+        region (ee.Geometry | None, optional): region to sample values for KMeans clustering, if set to `None` will use img.geometry(). default = None
+        samples (int, optional): number of stratified samples to gather for clustering from the initial water/no-water map. default=500
+        seed (int, optional): random number generator seed for sampling. default = 7
+        scale (int, optional): scale at which to perform reduction operations, setting higher will prevent OOM errors. default = 90
+        proba_threshold (float, optional): probability threshold to create a binary water mask, should be in range of 0-1. If None, then the probability values are returned. default = None
+    """
+
+    def _cluster_center(x):
+        return (
+            band_arr.mask(classes.eq(ee.Number(x))).reduce(ee.Reducer.mean(), [0])
+        ).get([0])
+
+    if region is None:
+        region = img.geometry()
+
+    if bands is None:
+        bands = img.bandNames()
+    else:
+        bands = ee.List(bands)
+
+    if rank_band is None:
+        rank_band = ee.String(bands.get(0))
+
+    samples = img.select(bands).sample(
+        region=region,
+        scale=scale,
+        numPixels=n_samples,
+        seed=seed,
+        dropNulls=True,
+        tileScale=16,
+    )
+
+    clusterer = ee.Clusterer.wekaXMeans(3, 12, 5).train(samples, bands)
+
+    samples = samples.cluster(clusterer, "init_classes")
+
+    classes = samples.aggregate_array("init_classes")
+    unique = classes.distinct().sort()
+    classes = ee.Array(classes)
+    band_arr = ee.Array(samples.aggregate_array(rank_band))
+
+    class_means = unique.map(_cluster_center)
+
+    if ranking == "min":
+        ranker = ee.Reducer.min()
+    elif ranking == "max":
+        ranker = ee.Reducer.max()
+    else:
+        raise NotImplementedError(
+            "ranking selection is not implemented. options are 'min' or 'max'"
+        )
+
+    ranked_mean = class_means.reduce(ranker)
+
+    water_class = class_means.indexOf(ranked_mean)
+
+    binary_samples = samples.map(
+        lambda x: (
+            ee.Feature(x).set(
+                "init_classes", ee.Number(x.get("init_classes")).eq(water_class)
+            )
+        )
+    )
+
+    classifier = (
+        ee.Classifier.smileRandomForest(numberOfTrees=50)
+        .setOutputMode("PROBABILITY")
+        .train(binary_samples, "init_classes", bands)
+    )
+
+    probas = img.select(bands).classify(classifier)
+
+    if proba_threshold is None:
+        output = probas.rename("water_proba")
+    else:
+        output = probas.gt(proba_threshold).rename("water").uint8()
+
+    return output
+
